/***************************************************************************
 * $Id: comp_proj.c,v 1.2 2000/11/02 18:17:51 frankw Exp $
 *
 * MODULE: 	libgrass (should go into libgis)
 * AUTHOR(S):	Frank Warmerdam - warmerda@home.com
 * PURPOSE: 	Code to compare projection definitions to see if they are
 *              equivelent.
 * 
 * COPYRIGHT:  	(C) 2000 by the GRASS Development Team
 *
 *   	    	This program is free software under the GPL (>=v2)
 *   	    	Read the file COPYING that comes with GRASS for details.
 *****************************************************************************
 *
 * $Log: comp_proj.c,v $
 * Revision 1.2  2000/11/02 18:17:51  frankw
 * Substantially improved projection comparison logic.
 *
 * Revision 1.1  2000/10/31 04:39:43  frankw
 * New
 *
 */

#include "libgrass.h"

/**********************************************************************
 * int G_compare_projections( proj_info1, proj_units1, 
 *                            proj_info2, proj_units2 )
 *
 * This function compares two projection/units definitions, and returns
 * 1 if they are equivelent, or 0 if they differ.  Currently the 
 * is inexact ... some differences will be missed, but it will be
 * improved in the future. 
 *
 * struct Key_Value *proj_info1
 *                         First projection definition as would be read
 *                         from the PROJ_INFO file (or NULL).
 * struct Key_Value *proj_units1
 *                         First units definition as would be read from the
 *                         PROJ_UNITS file (or NULL).
 * struct Key_Value *proj_info2
 *                         Second projection definition as would be read
 *                         from the PROJ_INFO file (or NULL).
 * struct Key_Value *proj_units2
 *                         Second units definition as would be read from the
 *                         PROJ_UNITS file (or NULL).
 *
 * Returns 0 if they differ, or 1 if the appear to be equivelent.
 */

int 
G_compare_projections( struct Key_Value *proj_info1, 
                       struct Key_Value *proj_units1, 
                       struct Key_Value *proj_info2, 
                       struct Key_Value *proj_units2 )

{
    char	*proj1, *proj2;
    /*
     * If they are both unreferenced, return a match.
     */
    if( proj_info1 == NULL && proj_info2 == NULL )
        return 1;

    /*
     * If only one is unreferenced, return failure.
     */
    if( proj_info1 == NULL || proj_info2 == NULL )
        return 0;
    
/* -------------------------------------------------------------------- */
/*      Verify that the linear unit translation to meters is OK.        */
/* -------------------------------------------------------------------- */
    if( proj_units1 != NULL && proj_units2 != NULL
        && G_find_key_value( "meter", proj_units1 ) != NULL
        && G_find_key_value( "meter", proj_units1 ) != NULL
        && atof(G_find_key_value( "meter", proj_units1 ))
           != atof(G_find_key_value( "meter", proj_units2 )) )
        return 0;

/* -------------------------------------------------------------------- */
/*      Are they both in the same projection?                           */
/* -------------------------------------------------------------------- */
    proj1 = G_find_key_value( "proj", proj_info1 );
    proj2 = G_find_key_value( "proj", proj_info2 );

    if( proj1 == NULL || proj2 == NULL )
    {
        G_warning( "G_compare_projections: got a proj_info Key_Value list\n"
                   "with no proj value.\n" );
        return 0;
    }

    if( G_strcasecmp(proj1,proj2) != 0 )
        return 0;

/* -------------------------------------------------------------------- */
/*      Do they have matching zones?  We should test for state plane    */
/*      too, but I am not sure how it is encoded.                       */
/* -------------------------------------------------------------------- */
    if( G_strcasecmp(proj1,"utm") == 0 )
    {
        char *zone1, *zone2;

        zone1 = G_find_key_value( "zone", proj_info1 );
        zone2 = G_find_key_value( "zone", proj_info2 );

        if( zone1 != NULL && zone2 != NULL && atoi(zone1) != atoi(zone2) )
            return 0;
    }

/* -------------------------------------------------------------------- */
/*      Do they have matching ellps codes?				*/
/* -------------------------------------------------------------------- */
    if( G_find_key_value("ellps",proj_info1) != NULL )
    {
        char *ellps1, *ellps2;

        ellps1 = G_find_key_value( "ellps", proj_info1 );
        ellps2 = G_find_key_value( "ellps", proj_info2 );

        if( ellps1 != NULL && ellps2 != NULL 
            && G_strcasecmp(ellps1,ellps2) != 0 )
            return 0;
    }

/* -------------------------------------------------------------------- */
/*      Add more details in later.                                      */
/* -------------------------------------------------------------------- */

    return 1;
}
