/******************************************************************************
 * $Id: elist.c,v 1.2 2000/11/02 16:38:08 frankw Exp $
 *
 * Project:  libgrass
 * Purpose:  Functions to fetch all files in the mapset path for a given
 *           element (such as cellhd), or all the mapsets, or locations
 *           under a given path.
 * Author:   Frank Warmerdam, warmerda@home.com
 *
 ******************************************************************************
 * Copyright (c) 2000, Frank Warmerdam
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 ******************************************************************************
 *
 * $Log: elist.c,v $
 * Revision 1.2  2000/11/02 16:38:08  frankw
 * fixed mixups with list_mapsets and list_locations
 *
 * Revision 1.1  2000/09/12 12:54:28  warmerda
 * New
 *
 */

/**********************************************************************
 *  char **G_fetch_list_element (element, mapset)
 *
 *      char *element    database element (eg, "cell", "cellhd", etc)
 *      char *mapset     mapset to be listed.
 *                       NULL to list all mapsets in mapset search list
 *                       "." will list current mapset
 *
 *   Fetches a list of all the files in the indicated element in the
 *   indicated mapsets.  The returned list is a NULL terminated array
 *   of strings that should be deallocated with G_free().
 *********************************************************************/

#include "libgrass.h"

#include <unistd.h>
#include <signal.h>
#include <dirent.h>

char **G_fetch_list_element (
    char *element,
    char *mapset )
{
    int n;
    char **list = NULL;

    for( n = 0; 1; n++ )
    {
        char buf[400];
        char *this_mapset;
        char **mapset_list;
        char path[1000];
        FILE *ls;
        FILE *G_popen();
        int  i;

        /*
         * if no specific mapset is requested, list the mapsets
         * from the mapset search list
         * otherwise just list the specified mapset
         */
        if( mapset == NULL || strcmp(mapset,"") == 0 )
            this_mapset = G__mapset_name(n);
        else if( n > 0 )
            this_mapset = NULL;
        else if( strcmp(mapset,".") == 0 )
            this_mapset = G_mapset();
        else
            this_mapset = mapset;
        
        if( this_mapset == NULL )
            break;

        G__file_name (path, element, "", this_mapset);
        if(access(path, 0) != 0)
            continue;

        /*
         * Capture the listing of this mapset element directory 
         * from the ls command.  Why not using opendir()/readdir()? 
         */

        mapset_list = G_fetch_list_files( path );
        for( i = 0; mapset_list != NULL && mapset_list[i] != NULL; i++ )
        {
            list = G_add_to_list( list, mapset_list[i] );
        }

        G_free_list( mapset_list );
    }

    return list;
}

/**********************************************************************
 *  char **G_fetch_list_locations( gisdbase )
 *
 *      char *gisdbase	Database to selection locations from, or 
 *                      NULL to use the default database.
 *
 * Returns a NULL terminated list of location names, which can be
 * freed with G_free_list() when no longer needed.
 *********************************************************************/

char **G_fetch_list_locations( char * gisdbase )

{
    if( gisdbase == NULL )
        gisdbase = G_gisdbase();

    return G_fetch_list_files( gisdbase );
}

/**********************************************************************
 *  char **G_fetch_list_mapsets( location_path )
 *
 *      char *location_path	full path to location to list mapsets for,
 *                              or NULL to use the default.
 *
 * Returns a NULL terminated list of mapset names, which can be
 * freed with G_free_list() when no longer needed.
 *********************************************************************/

char **G_fetch_list_mapsets( char * location_path )

{
    char	**list;

    if( location_path == NULL )
        location_path = G__location_path();
    else
        location_path = G_strdup(location_path);

    list = G_fetch_list_files( location_path );

    G_free( location_path );
    
    return list;
}

/**********************************************************************
 *  char **G_fetch_list_files( dir )
 *
 *      char *dir  	Directory to get the list of files from.
 *********************************************************************/

char **G_fetch_list_files( char * directory )

{
    DIR	*dir;
    char **list = NULL;
    struct dirent *entry;

    dir = opendir( directory );
    if( dir == NULL )
        return NULL;

    while( (entry = readdir(dir)) != NULL )
    {
        if( strcmp(entry->d_name,".") == 0 
            || strcmp(entry->d_name,"..") == 0 )
            continue;

        list = G_add_to_list( list, entry->d_name );
    }

    closedir( dir );

    return list;
}

/************************************************************************/
/*                           G_add_to_list()                            */
/************************************************************************/

char **G_add_to_list( char **list, char *new_item )

{
    int 	count;
    
    for( count = 0; list != NULL && list[count] != NULL; count++ ) {}

    if( list == NULL )
        list = (char **) G_malloc(sizeof(char *) * 2);
    else
        list = (char **) G_realloc(list, sizeof(char *) * (count+2) );

    list[count] = G_strdup(new_item);
    list[count+1] = NULL;

    return list;
}

/**********************************************************************
 *  void G_free_list( list )
 *
 *      char **list	NULL terminated list of strings to be freed with
 *                      G_free(), as returned by the various G_fetch_list*()
 *                      functions.
 *********************************************************************/

void G_free_list( char ** list )

{
    int 	i;

    if( list == NULL )
        return;

    for( i = 0; list[i] != NULL; i++ )
        G_free( list[i] );

    G_free( list );
}

