/***************************************************************************
 * $Id: get_proj4_def.c,v 1.3 2001/11/02 19:43:19 frankw Exp $
 *
 * MODULE: 	libgrass (should go into libgis)
 * AUTHOR(S):	Frank Warmerdam - warmerdam@pobox.com
 * PURPOSE:     Fetch projection definition as a pure PROJ.4 definition.
 * 
 * COPYRIGHT:  	(C) 2000 by the GRASS Development Team
 *
 *   	    	This program is free software under the GPL (>=v2)
 *   	    	Read the file COPYING that comes with GRASS for details.
 *****************************************************************************
 *
 * $Log: get_proj4_def.c,v $
 * Revision 1.3  2001/11/02 19:43:19  frankw
 * updated to latest GRASS 5 code and include files
 *
 * Revision 1.2  2000/11/02 18:20:03  frankw
 * added G_kv_to_proj4, trust G_get_projinfo
 *
 * Revision 1.1  2000/09/22 03:13:22  frankw
 * New
 *
 */

#include "libgrass.h"

#include <stdlib.h>

/***************************************************************************
 *  char *G_get_cell_as_proj4( cell, mapset );
 *
 *  char *cell           Name of cell to query.
 *  char *mapset         Name of mapset to query.
 *
 *  Returns a PROJ.4 definition of the coordinate system.  The returned
 *  string should be free with G_free() when no longer needed.  XY coordinate
 *  system, or other errors during processing will return in a return value
 *  of NULL. 
 **************************************************************************/

char *G_get_cell_as_proj4( char *cell, char *mapset )

{
    struct Key_Value *in_proj_info=NULL, *in_proj_units=NULL;
    struct Cell_head cellhd;
    char   *p4_def, *proj, path[1024];

    /* Note that this call may trigger a fatal error */
    G_get_cellhd( cell, mapset, &cellhd );
    /* Fetch the PROJ_INFO and PROJ_UNITS data if available. */
    
    in_proj_info = G_get_projinfo();
    in_proj_units = G_get_projunits();

    /*
     * Only use the LOCATION information, if it seems to match the cellhd.
     */
    if( in_proj_info != NULL )
        proj = G_find_key_value("proj", in_proj_info);
    else
        proj = NULL;

    if( proj != NULL 
        && ((cellhd.proj == PROJECTION_LL && G_strcasecmp(proj,"ll") == 0)
            || (cellhd.proj == PROJECTION_UTM && G_strcasecmp(proj,"utm")==0)
            || (cellhd.proj == PROJECTION_OTHER) ) )
    {
        /* FIXME: We should also check the state plane coordinate system,
           and the zones for utm coordinate systems. */

        p4_def = G_kv_to_proj4( in_proj_info, in_proj_units );
    }

    /*
     * Just generate the string from the information in the cellhd file.
     */
    else
    {
        if( cellhd.proj == PROJECTION_LL )
            p4_def = G_strdup("+proj=longlat +ellps=WGS84 " );
        else if( cellhd.proj == PROJECTION_UTM )
        {
            char	temp_def[128];

            sprintf( temp_def, "+proj=utm +zone=%d +ellps=WGS84 ",
                     cellhd.zone );
            p4_def = G_strdup(temp_def);
        }
        else
            p4_def = NULL;
    }

    if( in_proj_info != NULL )
        G_free_key_value( in_proj_info );
    if( in_proj_units != NULL )
        G_free_key_value( in_proj_units );

    return p4_def;
}

/***************************************************************************
 *  char *G_kv_to_proj4( in_proj_keys, in_units_keys );
 *
 *  struct Key_Value *in_proj_keys
 *                       List of projection keys, as returned by 
 *                       G_get_projinfo().
 *  struct Key_Value *in_units_keys
 *                       List of units keys, as returned by 
 *                       G_get_projunits().
 *
 *  Returns a PROJ.4 definition of the coordinate system.  The returned
 *  string should be free with G_free() when no longer needed.  Note that
 *  this function will return a PROJ.4.4.2 string, which includes support
 *  for the +proj=longlat.
 **************************************************************************/

char *G_kv_to_proj4( struct Key_Value *in_proj_keys, 
                     struct Key_Value *in_proj_units )

{
    char p4_def[2048];
    char *str, *str2;
    int i, k, south, nopt;

    /*
     * The following code is closely modelled on that in 
     * src/libes/proj/get_proj.c, including logic to convert
     * various GRASS specific parameters to generic PROJ.4 format.
     * For the most part the in_proj_keys is already in PROJ.4 format.
     * Note that get_proj() keeps units, and a bit of other stuff "outside"
     * the PJ structure in the pj_info structure.  We don't do that here.
     * We try to produce a PROJ.4 string that fully defines the coordinate
     * system.
     */

    p4_def[0] = '\0';

    /*
     * The name=Lat... seems to be some special way of making a coordinate
     * system lat/long.  Also, the existing code seems to treat an empty
     * projection as being lat/long.  How is XY handled?
     */
    str2 = G_find_key_value("name", in_proj_keys);
    str = G_find_key_value("proj", in_proj_keys);
    if( (str2 != NULL && strncmp(str2,"Lat",3) == 0)
        || (str != NULL && strncmp(str,"ll",2) == 0)
        || str == NULL || strlen(str) == 0 )
    {
        strcat( p4_def, "+proj=longlat " );
    }
    else
    {
        sprintf( p4_def+strlen(p4_def), "+proj=%s ", str );
    }

    /* 
     * Translate the zone, turning negative values to positive (but setting
     * the south flag). 
     */
    south = 0;
    str = G_find_key_value("zone", in_proj_keys);
    if (str != NULL && atoi(str) != 0 ) {
        sprintf( p4_def+strlen(p4_def), "+zone=%d ", abs(atoi(str)) );

        if( atoi(str) < 0 )
        {
            south = 1;
            strcat( p4_def, "+south " );
        }
    }

    /*
     * Handle the rest of the options.
     */
    nopt = in_proj_keys->nitems - 1;
    for (i = 1; i <= nopt; i++) {
        if (strncmp(in_proj_keys->key[i], "south", 5) == 0) {
            if( !south )
            {
                strcat( p4_def, "+south " );
                south = 1;
            }
        } else if ( strcmp(in_proj_keys->key[i],"proj") == 0 
                    || strcmp(in_proj_keys->key[i],"zone") == 0 
                    || strcmp(in_proj_keys->key[i],"name") == 0 ) {
            /* skip ... already handled */
        } else if( in_proj_keys->value[i] == NULL
                   || strlen(in_proj_keys->value[i]) == 0 ) {
            sprintf( p4_def+strlen(p4_def), "+%s ", in_proj_keys->key[i] );
        } else {
            sprintf( p4_def+strlen(p4_def), "+%s=%s ", 
                     in_proj_keys->key[i], in_proj_keys->value[i] );
        }
    }

    /*
     * Add the units conversion.
     */
    str = G_find_key_value("meters", in_proj_units);
    if (str != NULL) {
        sprintf( p4_def+strlen(p4_def), "+to_meter=%s ", str );
    }

    return G_strdup( p4_def );
}



/***************************************************************************
 *  int G_proj4_to_kv( proj4, out_proj_keys, out_units_keys );
 *
 *  char              *proj
 *                       PROJ.4 formatted string.
 *  struct Key_Value **out_proj_keys
 *                       List of projection keys suitable for G_set_projinfo().
 *  struct Key_Value **out_units_keys
 *                       List of units keys, suitable for G_set_projunits().
 *
 *  Translates a PROJ.4 coordinate system definition into GRASS keyword/value
 *  list format.  Note that the caller becomes responsible for deallocating
 *  the return Key_Value lists.  The function returns 0 on failure, and 1
 *  on success.
 **************************************************************************/

int G_proj4_to_kv( char * proj4, 
                   struct Key_Value **proj_info, 
                   struct Key_Value **proj_units )

{
    char	*pszRemaining;
    char        *units = NULL, *to_meter = NULL;

    *proj_info = NULL;
    *proj_units = NULL;

    /*
     * Return NULLes if we have no georeferencig info.
     */
    if( G_strstr(proj4,"proj=") == NULL )
        return 1;

    /*
     * Parse the PROJ.4 string into key/value pairs.  Do a bit of    
     *  extra work to "GRASSify" the result.
     */
    proj4 = G_strdup(proj4);
    *proj_info = G_create_key_value();
    pszRemaining = proj4;
    while( (pszRemaining = G_strstr((char *) pszRemaining,"+")) != NULL )
    {
        char 	*pszToken, *pszValue;

        pszRemaining++;
        
        /* Advance pszRemaining to end of this token[=value] pair */
        pszToken = (char *) pszRemaining;
        while( *pszRemaining != ' ' && *pszRemaining != '\0' )
            pszRemaining++;

        if( *pszRemaining == ' ' )
        {
            *pszRemaining = '\0';
            pszRemaining++;
        }

        /* parse token, and value */
        if( G_strstr(pszToken,"=") != NULL )
        {
            pszValue = G_strstr(pszToken,"=");
            *pszValue = '\0';
            pszValue++;
        }
        else
            pszValue = "";
        
        /* The latlong projection is known as ll in GRASS */
        if( G_strcasecmp(pszToken,"proj") == 0
            && G_strcasecmp(pszValue,"longlat") == 0 )
            pszValue = "ll";

        /* We will handle units separately */
        if( G_strcasecmp(pszToken,"to_meter") == 0 )
        {
            to_meter = G_strdup(pszValue);
            continue;
        }

        if( G_strcasecmp(pszToken,"units") == 0 )
        {
            units = G_strdup(pszValue);
            continue;
        }

        G_set_key_value( pszToken, pszValue, *proj_info );
    }

/* -------------------------------------------------------------------- */
/*      Set the linear units.                                           */
/* -------------------------------------------------------------------- */
    *proj_units = G_create_key_value();

    if( G_strstr(proj4,"longlat") != NULL || G_strstr(proj4,"latlong") != NULL 
        || G_strstr(proj4,"ll") != NULL )
    {
        /* We assume degrees ... someday we will be wrong! */
        G_set_key_value( "unit", "degree", *proj_units );
        G_set_key_value( "units", "degrees", *proj_units );
        G_set_key_value( "meter", "1.0", *proj_units );
    }
    else if( units == NULL && to_meter == NULL )
    {
        G_set_key_value( "unit", "meter", *proj_units );
        G_set_key_value( "units", "meters", *proj_units );
        G_set_key_value( "meter", "1.0", *proj_units );
    }
    else if( units == NULL && to_meter != NULL )
    {
        G_set_key_value( "unit", "unknown", *proj_units );
        G_set_key_value( "units", "unknowns", *proj_units );
        G_set_key_value( "meter", to_meter, *proj_units );
    }
    else if( G_strcasecmp(units,"km") )
    {
        G_set_key_value( "unit", "kilometer", *proj_units );
        G_set_key_value( "units", "kilometers", *proj_units );
        G_set_key_value( "meter", "1000.0", *proj_units );
    }
    else if( G_strcasecmp(units,"in") )
    {
        G_set_key_value( "unit", "inch", *proj_units );
        G_set_key_value( "units", "inches", *proj_units );
        G_set_key_value( "meter", "0.0254", *proj_units );
    }
    else if( G_strcasecmp(units,"ft") )
    {
        G_set_key_value( "unit", "foot", *proj_units );
        G_set_key_value( "units", "feet", *proj_units );
        G_set_key_value( "meter", "0.3048", *proj_units );
    }
    else if( G_strcasecmp(units,"yd") )
    {
        G_set_key_value( "unit", "yard", *proj_units );
        G_set_key_value( "units", "yards", *proj_units );
        G_set_key_value( "meter", "0.9144", *proj_units );
    }
    else if( G_strcasecmp(units,"us-ft") )
    {
        G_set_key_value( "unit", "usfoot", *proj_units );
        G_set_key_value( "units", "usfeet", *proj_units );
        G_set_key_value( "meter", "0.3048006", *proj_units );
    }
    else if( G_strcasecmp(units,"m") )
    {
        G_set_key_value( "meter", "meter", *proj_units );
        G_set_key_value( "meters", "meters", *proj_units );
        G_set_key_value( "meter", "1.0", *proj_units );
    }
    else
    {
        G_warning( "unrecognised units string `%s', assuming meters.", 
                   units );
        G_set_key_value( "meter", "meter", *proj_units );
        G_set_key_value( "meters", "meters", *proj_units );
        G_set_key_value( "meter", "1.0", *proj_units );
    }

    if( units != NULL )
        G_free( units );

    if( to_meter != NULL )
        G_free( to_meter );

    G_free( proj4 );

    return 1;
}



