#include "gis.h"
#include <unistd.h>

/*
 * This file contains modification that should eventually go back into
 * libgis in place of get_projinfo.c.
 *
 * The modification are that G_get_projunits() and G_get_projinfo() will
 * fallback to using the WIND file if the PROJ_INFO and PROJ_UNITS files
 * are not present.
 */

#define PERMANENT "PERMANENT"

struct Key_Value *
G_get_projunits() 
{
    int stat;
    struct Key_Value *in_units_keys = NULL;
    char path[1024];

    /*
     * If there is no PROJ_UNITS file, then make assumptions
     * based on the projection type or return NULL for ungeoreferenced.
     */
    G__file_name (path, "", UNIT_FILE, PERMANENT);
    if (access(path,0) != 0)
    {
        struct Key_Value *proj_info_keys;

        proj_info_keys = G_get_projinfo();
        if( proj_info_keys == NULL )
        {
            return NULL;
        }
        else if( G_find_key_value("proj",proj_info_keys) != NULL
                 && G_strcasecmp(G_find_key_value("proj",proj_info_keys),
                                 "ll") )
        {
            in_units_keys = G_create_key_value();
            G_set_key_value( "unit", "degree", in_units_keys );
            G_set_key_value( "units", "degrees", in_units_keys );
            G_set_key_value( "meter", "1.0", in_units_keys );
        }
        else
        {
            in_units_keys = G_create_key_value();
            G_set_key_value( "unit", "meter", in_units_keys );
            G_set_key_value( "units", "meters", in_units_keys );
            G_set_key_value( "meter", "1.0", in_units_keys );

        }
        
        if( proj_info_keys != NULL )
            G_free_key_value( proj_info_keys );

        return in_units_keys;
    }

    in_units_keys = G_read_key_value_file(path,&stat);
    if (stat != 0)
    { 
        G_warning( "ERROR in reading %s file for location %s\n",
                   UNIT_FILE, G_location());
        return NULL;
    }

    return in_units_keys;
}



struct Key_Value *
G_get_projinfo() 
{
    int stat;
    struct Key_Value *in_proj_keys = NULL;
    struct Cell_head cellhd;
    char path[1024];

    /*
     * Try to read the file.
     */
    G__file_name (path, "", PROJECTION_FILE, PERMANENT);
    if (access(path,0) == 0)
    {
        in_proj_keys = G_read_key_value_file(path,&stat);
        if (stat != 0)
        { 
            G_warning("ERROR in reading %s file for location %s\n",
                      PROJECTION_FILE, G_location());
            return NULL;
        }

        return in_proj_keys;
    }

    /* 
     * Try to get the WIND file, to setup projection parameters
     * based on it. 
     */

    G_get_window( &cellhd );
    if( cellhd.proj == PROJECTION_XY || cellhd.proj == PROJECTION_OTHER )
        return NULL;

    in_proj_keys = G_create_key_value();
    if( cellhd.proj == PROJECTION_UTM )
    {
        char	zone[32];
        
        sprintf( zone, "%d", cellhd.zone );
        G_set_key_value( "name", "UTM", in_proj_keys );
        G_set_key_value( "proj", "utm", in_proj_keys );
        G_set_key_value( "ellps", "wgs84", in_proj_keys );
        G_set_key_value( "a", "6378137.0", in_proj_keys );
        G_set_key_value( "es", "0.0066943800", in_proj_keys );
        G_set_key_value( "zone", zone, in_proj_keys );
    }
    else if( cellhd.proj == PROJECTION_LL )
    {
        G_set_key_value( "name", "Latitude-Longitude", in_proj_keys );
        G_set_key_value( "proj", "ll", in_proj_keys );
        G_set_key_value( "ellps", "wgs84", in_proj_keys );
        G_set_key_value( "a", "6378137.0", in_proj_keys );
        G_set_key_value( "es", "0.0066943800", in_proj_keys );
    }
    else if( cellhd.proj == PROJECTION_SP )
    {
        G_warning( "G_get_projinfo(): Haven't implement state plane support.");
    }
    else
    {
        /* this should never happen */
        return NULL;
    }

    return in_proj_keys;
}

