/***************************************************************************
 * $Id: gisinit2.c,v 1.7 2001/10/31 16:45:56 markus Exp $
 *
 * MODULE: 	libgrass (should go into libgis)
 * AUTHOR(S):	Frank Warmerdam - warmerda@home.com
 * PURPOSE: 	GRASS library initialization function allowing application
 *              setting of gisdbase, location and mapset.
 * 
 * COPYRIGHT:  	(C) 2000 by the GRASS Development Team
 *
 *   	    	This program is free software under the GPL (>=v2)
 *   	    	Read the file COPYING that comes with GRASS for details.
 *****************************************************************************
 *
 * $Log: gisinit2.c,v $
 * Revision 1.7  2001/10/31 16:45:56  markus
 * Markus Neteler: change of lzw to flate compression, addition of zlib in configure[.in] etc., additional check if libgrass is called by a web server, if so, read .grassrc5 from DOCUMENT_ROOT instead of /ssi0/ssi/neteler (gis/gisinit2.c)
 *
 * Revision 1.6  2000/11/20 21:20:01  frankw
 * Modified G_gisinit_2() to never call G_gisinit(), and to only
 * call G_no_gisinit() if gisinit hasn't been done once yet.  Thus
 * subsequent G_gisinit_2() calls just reset location, etc without
 * modifying the fileinfo or other G__ globals.
 *
 * Revision 1.5  2000/11/07 21:51:55  frankw
 * Don't put readonly strings in the environment
 *
 * Revision 1.4  2000/11/02 18:15:45  frankw
 * updated header, removed old comments
 *
 * Revision 1.3  2000/11/02 18:13:33  frankw
 * use G__setenv() instead of G_setenv() to avoid writing out .grassrc5
 *
 * Revision 1.2  2000/09/18 20:23:16  frankw
 * various fixes for case of missing /home/warmerda/.grassrc5
 *
 * Revision 1.1  2000/09/12 12:54:28  warmerda
 * New
 */

/***************************************************************************
 *  char **G_gisinit_2(pgm, gisdbase, location, mapset );
 *
 *      char *pgm	 name of the program for error reporting.
 *      char *gisdbase   path to database, or NULL to use GISDBASE environment
 *                       variable, or contents of ~/.grassrc file.
 *      char *location   location within database, or NULL to use environment
 *                       variable LOCATION_NAME or ~/.grassrc file.
 *      char *mapset     mapset within location, or NULL to use environment
 *                       variable MAPSET or ~/.grassrc.
 *                         
 * Library initialization similar to G_gisinit(), but giving the 
 * calling application some control over what database, location and
 * mapset is used. 
 *
 * If the initialization fails (because no valid mapset was given,
 * nor determinable) the function will return a non-zero error, otherwise
 * zero is returned. 
 *************************************************************************/

#include "gis.h"

#include <stdlib.h>

int G_gisinit_2( 
    char *pgm,
    char *gisdbase,
    char *location,
    char *mapset )

{
    G_set_program_name (pgm);
    
    if( !G__get_gisinit() )
        G_no_gisinit();

    /*
     * Set GISRC if not already set.
     */
    
    /* Are we a Web Server (Mapserver + GDAL ? */
    if( getenv("SERVER_SOFTWARE") != NULL )
    {
     /* yes, Apache calling... 
      * In this case put .grassrc5 into the DOCUMENT_ROOT directory
      */
     if( getenv("GISRC") == NULL )
     {
        char	gisrc[1500];

        sprintf( gisrc, "GISRC=%s/.grassrc5", getenv("DOCUMENT_ROOT") );
        putenv( G_strdup(gisrc) );
      }
    }
    else if( getenv("GISRC") == NULL )
     /* no, we are a normal user */
    {
        char	gisrc[1500];

        if( getenv("HOME") == NULL )
        {
            return 1;
        }

        sprintf( gisrc, "GISRC=%s/.grassrc5", getenv("HOME") );
        putenv( G_strdup(gisrc) );
    }
    
    /*
     * Set something for GISBASE, if not already set.
     */
    if( getenv("GISBASE") == NULL )
    {
        static char fake_gisbase[50];

        /* this odd approach is done to ensure that the string passed
         * to putenv() is not read-only, as some packages, notably the 
         * convertenviron() function of Python 1.5.2's posix module 
         * will temporarily alter strings in the environment.
         */
        strcpy( fake_gisbase, "GISBASE=/fake_gisbase" );
        putenv( fake_gisbase );
    }

    if( gisdbase != NULL )
        G__setenv( "GISDBASE", gisdbase );

    if( location != NULL )
        G__setenv( "LOCATION_NAME", location );
    
    if( mapset != NULL )
        G__setenv( "MAPSET", mapset );

    /* Verify that we have a valid location */
    if( G__getenv("LOCATION_NAME") == NULL )
        return 2;

    location = G__location_path();
    if( access(location,0) != 0 )
        return 2;

    /* verify that we have a valid mapset, and update permissions */
    mapset = G__mapset();
    if( mapset == NULL || G__mapset_permissions (G__mapset()) != 1 )
        return 3;

    return 0;
}



