/******************************************************************************
 * $Id: group2.c,v 1.2 2000/09/27 21:38:43 frankw Exp $
 *
 * Project:  libgrass
 * Purpose:  Extended support for group files.
 * Author:   Frank Warmerdam, warmerda@home.com
 *
 ******************************************************************************
 * Copyright (c) 2000, Frank Warmerdam
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 ******************************************************************************
 *
 * $Log: group2.c,v $
 * Revision 1.2  2000/09/27 21:38:43  frankw
 * Allow selection of the REF file for a group
 *
 * Revision 1.1  2000/09/26 17:37:55  frankw
 * New
 *
 */

#include "libgrass.h"
#include <stdlib.h>
#include <string.h>

/***************************************************************************
 *  int I_check_group( full_name, ret_mapset, ret_groupname );
 *
 *      char *full_name  Name of group to open.  This may be just the group
 *                       name, in which case it will be interpreted relative 
 *                       to the current mapset, or it may be a relative or 
 *                       absolute path to a group file in a group directory.
 *
 *      char **ret_mapset 
 *			 Location in which to return the mapset name, or NULL
 *                       if not requested.  Mapset name should be freed with
 *                       G_free() when no longer needed.
 *                         
 *      char **ret_groupname
 *			 Location in which to return the group name, or NULL
 *                       if not requested.  Group name should be freed with
 *                       G_free() when no longer needed.
 *                         
 * Returns 0 if the passed name does not appear to be a valid group name, or
 * 1 otherwise.   If this functions returns 1, then the groupname should
 * be suitable for I_get_group_ref(). This function may call G_gisinit_2() if 
 * the group is fully qualified and in a different location or database than 
 * the current one. 
 *
 * Applications should have called G_gisinit(), or G_gisinit_2() before
 * calling this function; however, this doesn't require that the 
 * initialization successfully identified a current grass mapset. 
 *
 * NOTE: I_check_group() will make the groups mapset the current mapset
 * for this process (by calling G__setenv("MAPSET",mapset)) if it isn't
 * already the current mapset.  This is different than G_check_cell(), and
 * is because the I_get_group_ref() API doesn't allow passing in a mapset). 
 *************************************************************************/

int I_check_group( char *group_name, char **ret_mapset, char **ret_group )

{
    char	*name_copy = NULL;
    char        *mapset = NULL;
    int         fd;

    if( strstr(group_name,"/") != NULL )
    {
        char    *location, *gisdb, *cellhd;
        
        name_copy = G_strdup( group_name );

        group_name = G_path_peel_element( name_copy );
        if( strcmp(group_name,"REF") == 0 )
            group_name = G_path_peel_element( name_copy );

        cellhd = G_path_peel_element( name_copy );
        if( cellhd == NULL || strcmp(cellhd,"group") != 0 )
        {
            G_free( name_copy );
            return 0;
        }
        
        /* TODO: we should add more validation of this as a path to a group
           in a valid grass database. */

        mapset = G_path_peel_element( name_copy );
        location = G_path_peel_element( name_copy );
        gisdb = name_copy;

        if( strlen(gisdb) == 0 || access(gisdb,0) != 0 )
        {
            G_free( name_copy );
            return 0;
        }

        /* Do we need to reinitialize? */
        if( G__getenv("GISDBASE") == NULL
            || strcmp(gisdb,G__getenv("GISDBASE")) != 0
            || G__getenv("LOCATION_NAME") == NULL
            || strcmp(location,G__getenv("LOCATION_NAME")) != 0 )
        {
            if( G_gisinit_2( G_program_name(), gisdb, location, mapset ) != 0 )
            {
                G_free( name_copy );
                return 0;
            }
        }

        if( G_strcasecmp(G_mapset(),mapset) != 0 )
            G__setenv("MAPSET", mapset);
    }
    else
        G__check_gisinit();

    if( mapset == NULL )
    {
        mapset = G_find_file2( "group", group_name, "" );
        if( mapset == NULL )
            return 0;
    }

    if( ret_mapset != NULL )
        *ret_mapset = G_strdup( mapset );

    if( ret_group != NULL )
        *ret_group = G_strdup( group_name );

    if( name_copy != NULL )
        G_free( name_copy );

    return 1;
}


