/***************************************************************************
 * $Id: make_loc.c,v 1.2 2000/11/02 18:05:04 frankw Exp $
 *
 * MODULE: 	libgrass (should go into libgis)
 * AUTHOR(S):	Frank Warmerdam - warmerda@home.com
 * PURPOSE: 	Function to create a new location automatically given a 
 *              "Cell_head", PROJ_INFO and PROJ_UNITS information.
 * 
 * COPYRIGHT:  	(C) 2000 by the GRASS Development Team
 *
 *   	    	This program is free software under the GPL (>=v2)
 *   	    	Read the file COPYING that comes with GRASS for details.
 *****************************************************************************
 * $Log: make_loc.c,v $
 * Revision 1.2  2000/11/02 18:05:04  frankw
 * added G_set_cellhd_from_projinfo
 *
 * Revision 1.1  2000/10/31 04:39:43  frankw
 * New
 */

#include "libgrass.h"

#include <stdlib.h>
#include <unistd.h>

/*
 * Returns 0 on success.
 * Returns -1 to indicate a system error (check errno).
 */
 

int G__make_location( 
    char *location_name,
    struct Cell_head *wind, 
    struct Key_Value *proj_info, 
    struct Key_Value *proj_units,
    FILE *report_file )

{
    char	path[2048];
    int         out_stat;

    /* Try to create the location directory, under the gisdbase. */
    sprintf( path, "%s/%s", G_gisdbase(), location_name );
    if( mkdir( path, 0775 ) != 0 )
        return -1;

    /* Make the PERMANENT mapset. */
    sprintf( path, "%s/%s/%s", G_gisdbase(), location_name, "PERMANENT" );
    if( mkdir( path, 0775 ) != 0 )
        return -1;

    /* make these the new current location and mapset */
    G__setenv( "LOCATION_NAME", location_name );
    G__setenv( "MAPSET", "PERMANENT" );

    /* Create the default, and current window files */
    G__put_window( wind, "", "DEFAULT_WIND" );
    G__put_window( wind, "", "WIND" );

    /* Write out the PROJ_INFO, and PROJ_UNITS if available. */
    if( proj_info != NULL )
    {
        G__file_name( path, "", "PROJ_INFO", "PERMANENT" );
        G_write_key_value_file( path, proj_info, &out_stat );
        if( out_stat != 0 )
            return -2;
    }

    if( proj_units != NULL )
    {
        G__file_name( path, "", "PROJ_UNITS", "PERMANENT" );
        G_write_key_value_file( path, proj_units, &out_stat );
        if( out_stat != 0 )
            return -2;
    }

    return 0;
}

/**********************************************************************
 * int G_make_location( location_name, wind, proj_info, proj_units, 
 *                      report_file )
 *
 * This function creates a new location in the current database,
 * initializes the projection, default window and current window.  
 *
 * char * location_name
 *                      The name of the new location.  Should not include
 *                      the full path, the location will be created within
 *                      the current database.
 * struct Cell_head *wind
 *                      Contains the default window setting for the
 *                      new location.  All fields should be set in this
 *                      structure, and care should be taken to ensure that
 *                      the proj/zone fields match the definition in the
 *                      proj_info parameter (see G_set_cellhd_from_projinfo()).
 *
 * struct Key_Value *proj_info
 *                      Projection definition suitable to write to the
 *                      PROJ_INFO file, or NULL for PROJECTION_XY.
 *
 * struct Key_Value *proj_units
 *                      Projection units suitable to write to the PROJ_UNITS
 *                      file, or NULL.
 *
 * FILE *report_file 
 *                      File to which creation information should be written
 *                      (can be stdout).  Currently not used.
 *
 * Returns 0 on success, or generates a fatal error on failure.  
 * The G__make_location() function operates the same, but returns a
 * non-zero error code on failure, instead of terminating. 
 *********************************************************************/

int G_make_location( 
    char *location_name,
    struct Cell_head *wind, 
    struct Key_Value *proj_info, 
    struct Key_Value *proj_units,
    FILE *report_file )

{
    int	err;

    err = G__make_location( location_name, wind, proj_info, proj_units, 
                            report_file );

    if( err == 0 )
        return 0;

    if( err == -1 )
    {
        perror( "G_make_location" );
    }

    G_fatal_error( "G_make_location failed." );
    
    return 0;
}

/**********************************************************************
 * void G_set_cellhd_from_projinfo( cellhd, proj_info )
 *
 * This function will set the proj and zone fields in a cell header 
 * structure based on the projection defined in the proj_list keyword
 * list.  The proj_info keyword list may be read using G_get_projinfo().
 * This function is most often used to initialize the wind parameter to
 * G_make_location().  Note that no fields in the cell header other than
 * proj and zone are modified.
 * 
 * struct Cell_head *cellhd
 *                      Structure in which the proj, and zone fields should
 *                      be set.
 *
 * struct Key_Value *proj_info
 *                      Keyword list from the PROJ_INFO file from which the
 *                      projection and zone should be derived.  May be NULL
 *                      for PROJECTION_XY.
 *********************************************************************/

void G_set_cellhd_from_projinfo( struct Cell_head *wind, 
                                struct Key_Value *proj_info )

{
    char	*proj, *zone;

    if( proj_info == NULL )
    {
        wind->proj = PROJECTION_XY;
        wind->zone = 0;
        return;
    }
    
    proj = G_find_key_value( "proj", proj_info );
    zone = G_find_key_value( "zone", proj_info );
    if( zone == NULL )
        zone = "";

    if( proj == NULL )
    {
        wind->proj = PROJECTION_XY;
        wind->zone = 0;
    }
    else if( G_strcasecmp(proj,"utm") == 0 )
    {
        wind->proj = PROJECTION_UTM;
        wind->zone = atoi(zone);
    }
    else if( G_strcasecmp(proj,"ll") == 0 )
    {
        wind->proj = PROJECTION_LL;
        wind->zone = 0;
    }
    else
    {
        wind->proj = PROJECTION_OTHER;
        wind->zone = 0;
    }
}
