/******************************************************************************
 * $Id: opencell2.c,v 1.2 2000/09/18 20:23:16 frankw Exp $
 *
 * Project:  libgrass
 * Purpose:  Entry points for opening a cell based on full filename, or
 *           relative to current mapset.  Generally used by non-GRASS 
 *           applications using libgrass.
 * Author:   Frank Warmerdam, warmerda@home.com
 *
 ******************************************************************************
 * Copyright (c) 2000, Frank Warmerdam
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 ******************************************************************************
 *
 * $Log: opencell2.c,v $
 * Revision 1.2  2000/09/18 20:23:16  frankw
 * various fixes for case of missing /home/warmerda/.grassrc5
 *
 * Revision 1.1  2000/09/12 12:54:29  warmerda
 * New
 *
 */

#include "libgrass.h"
#include <stdlib.h>
#include <string.h>

/***************************************************************************
 *  int G_check_cell( full_name, ret_mapset, ret_cellname );
 *
 *      char *full_name  Name of cell to open.  This may be just the cell name,
 *                       in which case it will be interpreted relative to the
 *                       current mapset, or it may be a relative or absolute
 *                       path to a cellhd file.
 *
 *      char **ret_mapset 
 *			 Location in which to return the mapset name, or NULL
 *                       if not requested.  Mapset name should be freed with
 *                       G_free() when no longer needed.
 *                         
 *      char **ret_cellname
 *			 Location in which to return the cell name, or NULL
 *                       if not requested.  Cell name should be freed with
 *                       G_free() when no longer needed.
 *                         
 * Returns 0 if the passed name does not appear to be a valid cell name, or
 * 1 otherwise.   If this functions returns 1, then the cell_name should
 * be suitable for G_open_cell_old_2() (but might not work with 
 * G_open_cell_old().  This function may call G_gisinit_2() if the cell
 * if fully qualified and in a different location or database than the current
 * one. 
 *
 * Applications should have called G_gisinit(), or G_gisinit_2() before
 * calling this function; however, this doesn't require that the 
 * initialization successfully identified a current grass mapset. 
 *************************************************************************/

int G_check_cell( char *cell_name, char **ret_mapset, char **ret_cell )

{
    char	*name_copy = NULL;
    char        *mapset = NULL;
    int         fd;

    if( strstr(cell_name,"/") != NULL )
    {
        char    *location, *gisdb, *cellhd;
        
        name_copy = G_strdup( cell_name );

        cell_name = G_path_peel_element( name_copy );
        cellhd = G_path_peel_element( name_copy );
        if( cellhd == NULL || strcmp(cellhd,"cellhd") != 0 )
        {
            G_free( name_copy );
            return 0;
        }
        
        /* TODO: we should add more validation of this as a path to a cellhd
           in a valid grass database. */

        mapset = G_path_peel_element( name_copy );
        location = G_path_peel_element( name_copy );
        gisdb = name_copy;

        if( strlen(gisdb) == 0 || access(gisdb,0) != 0 )
        {
            G_free( name_copy );
            return 0;
        }

        /* Do we need to reinitialize? */
        if( G__getenv("GISDBASE") == NULL
            || strcmp(gisdb,G__getenv("GISDBASE")) != 0
            || G__getenv("LOCATION_NAME") == NULL
            || strcmp(location,G__getenv("LOCATION_NAME")) != 0 )
        {
            if( G_gisinit_2( G_program_name(), gisdb, location, mapset ) != 0 )
            {
                G_free( name_copy );
                return 0;
            }
        }
    }
    else
        G__check_gisinit();

    if( mapset == NULL )
    {
        mapset = G_find_file2( "cell", cell_name, "" );
        if( mapset == NULL )
            return 0;
    }

    if( ret_mapset != NULL )
        *ret_mapset = G_strdup( mapset );

    if( ret_cell != NULL )
        *ret_cell = G_strdup( cell_name );

    if( name_copy != NULL )
        G_free( name_copy );

    return 1;
}

/***************************************************************************
 *  int G_open_cell_old_2( cell_name );
 *
 *      char *cell_name  Name of cell to open.  This may be just the cell name,
 *                       in which case it will be interpreted relative to the
 *                       current mapset, or it may be a relative or absolute
 *                       path to a cellhd file.
 *                         
 * Returns -1 if the open fails, or the file descriptor for the cell otherwise.
 * Applications should have called G_gisinit(), or G_gisinit_2() before
 * calling this function; however, this doesn't require that the 
 * initialization successfully identified a current grass mapset.  After
 * a successful return, the current database, location and mapset will have
 * been set suitably for the given cell.
 *************************************************************************/

int G_open_cell_old_2( char *full_name )

{
    char        *mapset = NULL, *cell_name = NULL;
    int         fd;

    if( !G_check_cell( full_name, &mapset, &cell_name ) )
        return -1;

    fd = G__open_cell_old( cell_name, mapset );

    G_free( mapset );
    G_free( cell_name );

    return fd;
}

/***************************************************************************
 *  char *G_path_peel_element( path );
 *
 *      char *path       Path from which a component should be extracted.
 *                       This string will be modified in place (shortened). 
 *                         
 * Returns the last component of the path, or NULL if the path is empty.
 * The returned string is a pointer into the original (modified) path, and
 * does not need to be freed. 
 *************************************************************************/

char *G_path_peel_element( char * path )

{
    int      i;

    if( path == NULL )
        return NULL;
    
    i = strlen(path)-1;
    if( i < 0 )
        return NULL;

    while( i >= 0 && path[i] != '/' )
        i--;

    if( i < 0 )
    {
        /* I hope we have one extra byte we can overwrite! */
        memmove( path+1, path, strlen(path)+1 );
        path[0] = '\0';
        return path+1;
    }
    else
    {
        path[i] = '\0';
        return path + i + 1;
    }
}



